/**
 * # DZM Countdown
 * @description
 * Creates and displays a countdown with several options.
 * Use for single countdown or recurring countdowns with different dates for each weekday.
 *
 * @usage
 * 1. Create an element with a data attribute `data-dzm-ctd-injector` and set an ID as its value.
 * 2. Create a script that instantiates a new `DZMCountdown` with the same ID in its constructor call,
 *    and execute the `init` function with all the required options.
 *
 * @example
 * <div data-dzm-ctd-injector="my-id"></div>
 *
 * <script>
 *      (new DZMCountdown('my-id')).init({
 *          // Add your countdown options here
 *      });
 * </script>
 *
 * @param uid unique id for this countdown - you can use multiple containers with same id, if you want to display the same countdown several times
 */
class DZMCountdown{



    constructor(uid) {
        this.uid = uid

        this.mode = 'repeat' // repeat|single

        this.singleTargetTime = null

        this.now = new Date()
        this.start = new Date()
        
        this.useOnMonday = false
        this.useOnTuesday = false
        this.useOnWednesday = false
        this.useOnThursday = false
        this.useOnFriday = false
        this.useOnSaturday = false
        this.useOnSunday = false   
        
        this.mondayTargetTime = null
        this.tuesdayTargetTime = null
        this.wednesdayTargetTime = null
        this.thursdayTargetTime = null
        this.fridayTargetTime = null
        this.saturdayTargetTime = null
        this.sundayTargetTime = null

        this.holidays = []

        this.countdownStyle = 'digital' // full|short|digital|onlyDate
        this.expiredStyle = 'hide' // hide|altText|nextDate|nextCountdown

        this.preserveExpiredValues = false

        this.icon = 'fas fa-bolt'
        this.iconColor = 'inherit'
        this.altIcon = 'fas fa-fire'
        this.altIconColor = 'inherit'
        this.expiredIcon = 'fas fa-star'
        this.expiredIconColor = 'inherit'
        this.countdownColor = 'inherit'
        this.countdownExpiredColor = 'inherit'

        this.countdownTextBefore = 'Bestelle innerhalb von '
        this.countdownTextAfter = 'und wir liefern so schnell wie möglich'
        this.expiredTextBefore = 'Bestelle bis '
        this.expiredTextAfter = ' und deine Ware kriegt das nächste Schiff'
        this.altText = 'Schade, leider verpasst'

        this.textColor = 'inherit'
        this.altTextColor = 'inherit'
        this.expiredTextColor = 'inherit'

        this.mainWidgetStyle = 'dzm-notice-clean'
        this.altTextWidgetStyle = 'dzm-notice-clean'
        this.expiredCountdownWidgetStyle = 'dzm-notice-clean'

        this.strings = {
            day:'Tag',
            days: 'Tage',
            hour:'Stunde',
            hours:'Stunden',
            minute:'Minute',
            minutes:'Minuten',
            second:'Sekunde',
            seconds:'Sekunden',
            today:'heute',
            tomorrow:'morgen',
            until:'zum',
            clock:'Uhr'

        }


        this.nextTargetDate = null
        this.isExpired = true

        this.interval = null

        this.injectors = []
        this.selectors={
            injectorSelector: `[data-dzm-ctd-injector="${uid}"]`
        }

        // debug
        this.debug = false
        this.debugStopTimer = false;
        this.debugTimeFactor = 0;

        this.runTimer = true
        this.pauseTime = new Date()
        this.offset = 0



    }

    /**
     * Initializes and controlls appearence of countdown and starts timer
     * @param options Options for setting up Countdown:
     *
     * @usage
     * ## General
     *
     * | Option           | Mandatory or optional | type   | possible values                                                                 |
     * |------------------|-----------------------|--------|---------------------------------------------------------------------------------|
     * | mode             | M                     | string | repeat, single                                                                  |
     * | countdownStyle   | M                     | string | full, short, digital, digitalWithDays, onlyValues, onlyValuesWithDays, onlyDate |
     * | expiredStyle     | M                     | string | hide, altText, nextDate, nextCountdown                                          |
     *
     *
     * ## Target Time (when mode is 'repeat')
     *
     * | Option                | Mandatory or optional | type   | possible values          |
     * |-----------------------|-----------------------|--------|--------------------------|
     * | mondayTargetTime      | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     * | tuesdayTargetTime     | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     * | wednesdayTargetTime   | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     * | thursdayTargetTime    | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     * | fridayTargetTime      | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     * | saturdayTargetTime    | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     * | sundayTargetTime      | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     *
     *
     * ## Target Time (when mode is 'single')
     * | Option                | Mandatory or optional | type   | possible values          |
     * |-----------------------|-----------------------|--------|--------------------------|
     * | singleTargetTime      | M                     | string | yyyy-MM-ddThh:mm.ss.000Z |
     */
    init(options){
        if(options === null){
            console.error('NO OPTIONS SET')
            return;
        }
        // GET CONTAINERS

        // GET INJECTOR
        this.injectors = document.querySelectorAll(this.selectors.injectorSelector)
        if(options.debug){
            console.log('OPTIONS',options)
            this.debug = options.debug
        }
        // CHECK FOR CONTAINERS
        if(this.injectors.length <=0){
            console.error('NO COUNTDOWN INJECTOR CONTAINER FOUND')
        }

        // TODAYs DATE
        //let now = new Date('2024-01-04T15:00:00.000Z')
        let now = new Date()
        // BASE SETTINGS
        this.countdownStyle = options.countdownStyle || this.countdownStyle
        this.expiredStyle = options.expiredStyle || this.expiredStyle
        this.mode = options.mode || this.mode

        this.holidays = options.holidays||[]

        switch (this.mode){
            case 'single':

                if(!('singleTargetTime' in options)){
                    console.error('COUNTDOWN::mode set to single but singleTargetTime missing',options)
                    return;
                }
                this.singleTargetTime = options.singleTargetTime // MAYBE USELESS
                this.nextTargetDate = new Date(options.singleTargetTime)

                break;
            case 'repeat':
                let dateExists = false
                if ('mondayTargetTime' in options) {
                    this.mondayTargetTime = options.mondayTargetTime
                    this.useOnMonday = true
                    dateExists = true
                }
                if ('tuesdayTargetTime' in options) {
                    this.tuesdayTargetTime = options.tuesdayTargetTime;
                    this.useOnTuesday = true;
                    dateExists = true;
                }
                if ('wednesdayTargetTime' in options) {
                    this.wednesdayTargetTime = options.wednesdayTargetTime;
                    this.useOnWednesday = true;
                    dateExists = true;
                }
                if ('thursdayTargetTime' in options) {
                    this.thursdayTargetTime = options.thursdayTargetTime;
                    this.useOnThursday = true;
                    dateExists = true;
                }
                if ('fridayTargetTime' in options) {
                    this.fridayTargetTime = options.fridayTargetTime;
                    this.useOnFriday = true;
                    dateExists = true;
                }
                if ('saturdayTargetTime' in options) {
                    this.saturdayTargetTime = options.saturdayTargetTime;
                    this.useOnSaturday = true;
                    dateExists = true;
                }
                if ('sundayTargetTime' in options) {
                    this.sundayTargetTime = options.sundayTargetTime;
                    this.useOnSunday = true;
                    dateExists = true;
                }

                if(!dateExists){
                    console.error('COUNTDOWN::mode set to repeat but no targetTimes set', options);
                    return;
                }
                if(this.debug)console.log('NOW', now);
                let nextTargetDay = this.getNextShippingDay(now);
                if(this.debug)console.log('NEXT TARGET DAY', nextTargetDay);
                this.nextTargetDate = this.buildTargetDate(nextTargetDay);
                if(this.debug)console.log('NEXT TARGET DATE', this.nextTargetDate);

                let diff = this.nextTargetDate - now;

                if(this.debug) console.log('4 :: diff', diff);
                if(this.debug)console.log('NEXT TARGET DATE',this.nextTargetDate);

                if(diff <= 0){
                    this.isExpired = true;
                    this.nextTargetDate.setDate(this.nextTargetDate.getDate()+1);
                    if(this.debug)console.log('SET DATE TO NEXT TARGET DATE +1', this.nextTargetDate);
                    nextTargetDay = this.getNextShippingDay(this.nextTargetDate);
                    this.nextTargetDate = this.buildTargetDate(nextTargetDay);
                    if(this.debug)console.log('REPEATED NEXT TARGET DATE',this.nextTargetDate);

                    diff = this.nextTargetDate - now;

                }
                break;
            default:
                console.error('unknown mode', this.mode);
                return;

        }



        // SETUP TEXTS AND COLORE

        this.icon = options.icon ||'';
        this.iconColor = options.iconColor ||this.iconColor;
        this.altIcon = options.altIcon ||'';
        this.altIconColor = options.altIconColor ||this.altIconColor;
        this.expiredIcon = options.expiredIcon ||'';
        this.expiredIconColor = options.expiredIconColor ||this.expiredIconColor;
        this.countdownColor = options.countdownColor ||this.countdownColor;
        this.countdownExpiredColor = options.countdownExpiredColor ||this.countdownExpiredColor;

        this.countdownTextAfter = options.countdownTextAfter ||'';
        this.countdownTextBefore = options.countdownTextBefore ||'';
        this.expiredTextBefore = options.expiredTextBefore ||'';
        this.expiredTextAfter = options.expiredTextAfter ||'';
        this.altText = options.altText ||'';

        this.textColor = options.textColor ||this.textColor;
        this.altTextColor = options.altTextColor ||this.altTextColor;
        this.expiredTextColor = options.expiredTextColor ||this.expiredTextColor;

        this.mainWidgetStyle = options.mainWidgetStyle || this.mainWidgetStyle;
        this.altTextWidgetStyle = options.altTextWidgetStyle || this.altTextWidgetStyle;
        this.expiredCountdownWidgetStyle = options.expiredCountdownWidgetStyle || this.expiredCountdownWidgetStyle;

        this.strings = {
            day:options.strings.day||this.strings.day,
            days: options.strings.days||this.strings.days,
            hour:options.strings.hour||this.strings.hour,
            hours:options.strings.hours||this.strings.hours,
            minute:options.strings.minute||this.strings.minute,
            minutes:options.strings.minutes||this.strings.minutes,
            second:options.strings.second||this.strings.second,
            seconds:options.strings.seconds||this.strings.seconds,
            today:options.strings.today||this.strings.today,
            tomorrow:options.strings.tomorrow||this.strings.tomorrow,
            until:options.strings.until||this.strings.until,
            clock:options.strings.clock||this.strings.clock,

        };

        this.preserveExpiredValues = options.preserveExpiredValues || false;

        this.debugStopTimer = options.debugStopTimer || false;
        this.debugTimeFactor = options.debugTimeFactor || 0;
        this.debug = options.debug || false;

        if(this.debug){
            console.error('DZMCountdown::DEBUG MODE IS ACTIVE');

            this.offset = options.debugInitialOffset || 0;

            if(!('countdownInstances' in window)){
                window.countdownInstances = {}
            }
            window.countdownInstances[this.uid]=this;

        }

        if(this.debug)console.log('run');
        // RUN COUNTDOWN
        this.run();

        
    }
    pause(){
        this.runTimer = false;
        this.pauseTime = new Date();
    }
    resume(){
        this.runTimer = true;
    }

    stop(){
        clearInterval(this.interval);
    }
    offsetUp(factor = 1){
        this.offset += 1000 * factor;

    }
    offsetDown(factor = 1){
        this.offset -= 1000 * factor;
    }

    offsetReset (){
        this.offset = 0;
    }

    run(){
        let k = 0;
        this.interval = setInterval(()=>{

            if(this.debug){
                k++;
                console.log(k);
            }




            let now = this.runTimer ? new Date() : this.pauseTime;
            if(this.debug)console.log('NOW', now);
            let nowDiff = 0;
            if(this.debugTimeFactor > 0){
                nowDiff = (now - this.start * this.debugTimeFactor);
            }
            let output = '';

            let diff =  (this.nextTargetDate - now) - nowDiff + this.offset;
            if(this.debug)console.log('DIFF', diff);

            if(diff <= 0 && this.mode === 'repeat'){
                this.nextTargetDate.setDate(this.nextTargetDate.getDate()+1);
                let nextTargetDay = this.getNextShippingDay(this.nextTargetDate);
                this.nextTargetDate = this.buildTargetDate(nextTargetDay);

                diff = (this.nextTargetDate - now)  - nowDiff + this.offset;
                this.isExpired = true;


            }
            const seconds = Math.floor((diff / 1000) % 60);
            const minutes = Math.floor((diff / 1000 / 60) % 60);
            const hours = Math.floor((diff / (1000 * 60 * 60)) % 24);
            const totalHours = Math.floor(diff / (1000 * 60 * 60));
            const days = Math.floor(diff / (1000 * 60 * 60 * 24));

            if(diff <= 0){

                // TODO: HANDLE EXPIRED
                clearInterval(this.interval);
                this.isExpired = true;


            }else{
                if(this.mode !== 'single'){
                    let tomorrow = new Date();
                    tomorrow.setDate(now.getDate() + 1);

                    if(days > 0 || (this.nextTargetDate.getDate() > now.getDate() || tomorrow.getDate() === 1)){
                        this.isExpired = true;
                        if(this.debug)console.log('expired',days,this.nextTargetDate.getDate(), now.getDate());
                    }else{
                        if(this.debug)console.log('in time',days,this.nextTargetDate.getDate(), now.getDate());

                        this.isExpired = false;
                    }
                }else{
                    this.isExpired = false
                }

            }


            if(this.isExpired){

                if(this.debug)console.log('expired', this.expiredStyle);
                switch( this.expiredStyle ){
                    case 'hide':
                        clearInterval(this.interval);
                        this.injectors.forEach(i=>{
                            if(i.classList.value.includes(this.mainWidgetStyle)){
                                i.classList.remove(...this.mainWidgetStyle.split(' '));
                            }
                            i.style.display = 'none';
                        })
                        break;
                    case 'altText':
                        clearInterval(this.interval)
                        output = this.getAltString();
                        this.injectors.forEach(i=>{
                            i.innerHTML = output;
                            if(i.classList.value.includes(this.mainWidgetStyle)){
                                i.classList.remove(...this.mainWidgetStyle.split(' '));
                            }
                            if(!i.classList.value.includes(this.altTextWidgetStyle)){
                                i.classList.add(...this.altTextWidgetStyle.split(' '));
                            }
                        })
                        break;
                    case 'nextDate':
                        if(this.debug)console.log('nextCountdown');
                        clearInterval(this.interval);
                        if(this.mode === 'single'){
                            console.warn('ExpiredStyle "nextDate" not allowed in single mode');
                            this.injectors.forEach(i=>{
                                i.style.display = 'none';
                                if(i.classList.value.includes(this.mainWidgetStyle)){
                                    i.classList.remove(...this.mainWidgetStyle.split(' '));
                                }
                            })
                        }
                        output = this.getTimeString(days,hours,totalHours,minutes,seconds);
                        this.injectors.forEach(i=>{
                            i.innerHTML = output;
                            if(i.classList.value.includes(this.mainWidgetStyle)){
                                i.classList.remove(...this.mainWidgetStyle.split(' '));
                            }
                            if(!i.classList.value.includes(this.expiredCountdownWidgetStyle)){
                                i.classList.add(...this.expiredCountdownWidgetStyle.split(' '));
                            }
                        })
                        break;
                    case 'nextCountdown':
                        if(this.debug)console.log('nextCountdown');
                        if(this.mode === 'single'){
                            console.warn('ExpiredStyle "nextCountdown" not allowed in single mode');
                            this.injectors.forEach(i=>{
                                i.style.display = 'none';
                                if(i.classList.value.includes(this.mainWidgetStyle)){
                                    i.classList.remove(...this.mainWidgetStyle.split(' '));
                                }
                            })
                        }
                        output = this.getTimeString(days,hours,totalHours,minutes,seconds);
                        this.injectors.forEach(i=>{
                            i.innerHTML = output;
                            if(i.classList.value.includes(this.mainWidgetStyle)){
                                i.classList.remove(...this.mainWidgetStyle.split(' '));
                            }
                            if(!i.classList.value.includes(this.expiredCountdownWidgetStyle)){
                                i.classList.add(...this.expiredCountdownWidgetStyle.split(' '));
                            }
                        })
                        break;
                    default:
                        console.error('INVALID EXPIRED STYLE',this.expiredStyle);
                        clearInterval(this.interval);
                        this.injectors.forEach(i=>{
                            i.style.display = 'none';
                            if(i.classList.value.includes(this.mainWidgetStyle)){
                                i.classList.remove(...this.mainWidgetStyle.split(' '));
                            }
                        });
                }
            }else{
                output = this.getTimeString(days,hours,totalHours,minutes,seconds)
                this.injectors.forEach(i=>{
                    i.innerHTML = output
                    if(i.classList.value.includes(this.altTextWidgetStyle)){
                        i.classList.remove(...this.altTextWidgetStyle.split(' '));
                    }
                    if(i.classList.value.includes(this.expiredCountdownWidgetStyle)){
                        i.classList.remove(...this.expiredCountdownWidgetStyle.split(' '));
                    }
                    if(!i.classList.value.includes(this.mainWidgetStyle)){
                        i.classList.add(...this.mainWidgetStyle.split(' '));
                    }
                })
            }
            if(this.countdownStyle === 'onlyDate'){
                clearInterval(this.interval);

            }

            if(this.debugStopTimer){
                console.error('DEBUG STOP TIMER MODE IS ACTIVE - INTERVAL CLEARED');
                clearInterval(this.interval);
            }


        },500)
    }

    getNextShippingDay(targetDate){
        if(this.debug)console.log('getNextShippingDay',targetDate);
        let workingDate = new Date(targetDate.toISOString());
        if(this.debug)console.log('workingDate',workingDate);
        if(this.debug)console.log('workingDate.getDay()',workingDate.getDay());

        let k = 90;
        let cDate = new Date();

        while(true){
            k--;
            if(this.holidays.includes(workingDate.toISOString().slice(0,10))){
                workingDate.setDate(workingDate.getDate() + 1);
                continue;
            }
            switch (workingDate.getDay()) {
                case 0:
                    if (this.useOnSunday )
                        return workingDate;
                    break;
                case 1:
                    if (this.useOnMonday )
                        return workingDate;
                    break;
                case 2:
                    if (this.useOnTuesday ){
                        return workingDate;
                    }
                    break;
                case 3:
                    if (this.useOnWednesday )
                        return workingDate;
                    break;
                case 4:
                    if (this.useOnThursday )
                        return workingDate;
                    break;
                case 5:
                    if (this.useOnFriday )
                        return workingDate;
                    break;
                case 6:
                    if (this.useOnSaturday )
                        return workingDate;
                    break;


            }
            workingDate.setDate(workingDate.getDate() + 1);
            if(k<=0)
                return workingDate;
        }
    }
    isDST(d) {
        let jan = new Date(d.getFullYear(), 0, 1).getTimezoneOffset();
        //console.log(jan)
        let jul = new Date(d.getFullYear(), 6, 1).getTimezoneOffset();
        console.log(jul)
        console.log(d.getTimezoneOffset())
        return Math.max(jan, jul) !== d.getTimezoneOffset();
    }
    buildTargetDate(targetDate){
        console.warn('BUILD TAGET DATE START::TARGETDATE',targetDate)
        //targetDate = new Date(targetDate.getTime() - (targetDate.getTimezoneOffset() * 60000));
        console.warn('BUILD TAGET DATE START::TARGETDATE',targetDate)

        let targetDay = targetDate.toISOString().slice(0,10);


        switch (targetDate.getDay()) {
            case 0:
                targetDay += 'T' + this.sundayTargetTime.slice(11,24);
                break;
            case 1:
                targetDay += 'T' + this.mondayTargetTime.slice(11,24);
                break;
            case 2:
                targetDay += 'T' + this.tuesdayTargetTime.slice(11,24);
                break;
            case 3:
                targetDay += 'T' + this.wednesdayTargetTime.slice(11,24);
                break;
            case 4:
                targetDay += 'T' + this.thursdayTargetTime.slice(11,24);
                break;
            case 5:
                targetDay += 'T' + this.fridayTargetTime.slice(11,24);
                break;
            case 6:
                targetDay += 'T' + this.saturdayTargetTime.slice(11,24);
                break;
        }
        console.warn('TARGETDAY string',targetDay)
        targetDay = new Date(targetDay)
        console.warn('TARGETDAY date',targetDay)
        if(this.isDST(targetDay)){
            console.warn('isDST')
            targetDay = new Date(targetDay.getTime() - 3600000)
        }else{
            console.warn('is not DST')
        }
        console.warn('TARGETDAY date after',targetDay)


        return targetDay
    }

    getTimeString(days, hours,totalHours, minutes, seconds){

        let string = '';
        if(this.isExpired){
            string = (this.expiredIcon !== '' ? `<i class="${this.expiredIcon} expired" style="color:${this.expiredIconColor};"></i>`  : '') + `<span class="dzm-countdown-text expired"  style="color:${this.expiredTextColor};">${this.expiredTextBefore} <span class="dzm-countdown-time expired" style="color:${this.countdownExpiredColor};">`;
        }else{
            string = (this.icon !== '' ? `<i class="${this.icon}" style="color:${this.iconColor};"></i>`  : '') + `<span class="dzm-countdown-text"  style="color:${this.textColor};">${this.countdownTextBefore} <span class="dzm-countdown-time" style="color:${this.countdownColor};">`;

        }
        let style = this.countdownStyle;
        if(this.isExpired && this.expiredStyle === 'nextDate')
            style = 'onlyDate';
        switch (style){
            case 'full':
                // days
                string += (days>0?`<span class="dzm-countdown-time-days">${days} ${(days>1 ? this.strings.days : this.strings.day)}</span>&nbsp;`:(this.preserveExpiredValues ? `<span class="dzm-countdown-time-days">0 ${this.strings.days}</span>&nbsp;`:''));
                // hours
                string += (days<=0 && hours <= 0?(this.preserveExpiredValues ? `<span class="dzm-countdown-time-hours">0 ${this.strings.hours}</span>&nbsp;`:''):`<span class="dzm-countdown-time-hours">${hours} ${(hours===1 ? this.strings.hour : this.strings.hours)}</span>&nbsp;`);
                // minutes
                string += (days<=0 && hours <= 0 && minutes <= 0?(this.preserveExpiredValues ? `<span class="minutes">0 ${this.strings.minutes}</span>&nbsp;`:''):('<span class="dzm-countdown-time-minutes">' + minutes + ' ' + (minutes===1 ? this.strings.minute : this.strings.minutes) + '</span>&nbsp;'));
                // seconds
                string += ('<span class="dzm-countdown-time-seconds">' + seconds + ' ' + (seconds===1 ? this.strings.second : this.strings.seconds) + '</span>');
                break;
            case 'short':
                string += (days>0?`<span class="dzm-countdown-time-days">${days}d</span>&nbsp;`:(this.preserveExpiredValues ? `<span class="dzm-countdown-time-days">0d</span>&nbsp;`:''));
                string += `<span class="dzm-countdown-time-hours">${hours}h</span>&nbsp;`;
                string += `<span class="dzm-countdown-time-minutes">${minutes}m</span>&nbsp;`;
                string += `<span class="dzm-countdown-time-seconds">${seconds}s</span>`;
                break;
            case 'digital':
                string += `<span class="dzm-countdown-time-hours">${totalHours.toString().padStart(2, '0')}</span>`;
                string += ':';
                string += `<span class="dzm-countdown-time-minutes">${minutes.toString().padStart(2, '0')}</span>`;
                string += ':';
                string +=`<span class="dzm-countdown-time-seconds">${seconds.toString().padStart(2, '0')}</span>`;

                break;
            case 'digitalWithDays':
                string += (days>0?`<span class="dzm-countdown-time-days">${days}d</span>&nbsp;`:(this.preserveExpiredValues ? `<span class="dzm-countdown-time-days">0d</span>&nbsp;`:''));
                string += `<span class="dzm-countdown-time-hours">${hours.toString().padStart(2,'0')}</span>`;
                string += ':';
                string += `<span class="dzm-countdown-time-minutes">${minutes.toString().padStart(2, '0')}</span>`;
                string += ':';
                string += `<span class="dzm-countdown-time-seconds">${seconds.toString().padStart(2, '0')}</span>`;

                break;
            case 'onlyValues':
                string += (totalHours>0?`<span class="dzm-countdown-time-hours">${totalHours.toString().padStart(2, '0')}</span>` : (this.preserveExpiredValues ? `<span class="dzm-countdown-time-hours">00</span>` : '') );
                string += (days<=0 && hours <= 0 && minutes <= 0?(this.preserveExpiredValues ? `<span class="dzm-countdown-time-minutes">00</span>` : ''):(`<span class="dzm-countdown-time-minutes">${minutes.toString().padStart(2, '0')}</span>`));
                string += `<span class="dzm-countdown-time-seconds">${seconds.toString().padStart(2, '0')}</span>`;
                break;
            case 'onlyValuesWithDays':
                string += (days>0?`<span class="dzm-countdown-time-days">${days.toString().padStart(2, '0')}</span>`:(this.preserveExpiredValues ?`<span class="dzm-countdown-time-days">00</span>`:''));
                string += (days<=0 && hours <= 0? (this.preserveExpiredValues ? `<span class="dzm-countdown-time-hours">00</span>` : '') : `<span class="dzm-countdown-time-hours">${hours.toString().padStart(2, '0')}</span>` );
                string += (days<=0 && hours <= 0 && minutes <= 0?(this.preserveExpiredValues ? `<span class="dzm-countdown-time-minutes">00</span>` : ''):(`<span class="dzm-countdown-time-minutes">${minutes.toString().padStart(2, '0')}</span>`));
                string += `<span class="dzm-countdown-time-seconds">${seconds.toString().padStart(2, '0')}</span>`;
                break;
            case 'onlyDate':
                let today = new Date();
                let tomorrow = new Date();
                tomorrow.setDate(tomorrow.getDate() + 1);
                if(this.debug)console.log('TODAY +1',today.getDate()+1);
                if(this.debug)console.log('nextTargetDate.getDate()',this.nextTargetDate.getDate());
                if(today.getDate() === this.nextTargetDate.getDate()){
                    string += `${this.strings.today}&nbsp;`;
                }else if(tomorrow.getDate() === this.nextTargetDate.getDate()){

                    string += `${this.strings.tomorrow}&nbsp;`;
                }else{
                    string += `${this.strings.until} ${this.nextTargetDate.getDate()}.${(this.nextTargetDate.getMonth() +1)}&nbsp;`;
                }

                string += `${this.nextTargetDate.getHours().toString().padStart(2, '0')}:${this.nextTargetDate.getMinutes().toString().padStart(2, '0')} ${this.strings.clock}`;
        }
        if(this.isExpired){
            string += `</span>&nbsp;${this.expiredTextAfter}</span>`;
        }else{
            string += `</span>&nbsp;${this.countdownTextAfter}</span>`;

        }


        return string;

    }

    getAltString(){
        let string = '';
        string = (this.altIcon !== '' ? `<i class="${this.altIcon} alt" style="color:${this.altIconColor};"></i>`  : '') + `<span class="dzm-countdown-text alt" style="color:${this.altTextColor};">${this.altText}</span>`;

        return string;


    }
    
}